/*
    Copyright © 2022, Inochi2D Project
    Distributed under the 2-Clause BSD License, see LICENSE file.
    
    Authors: Luna Nielsen
*/
module inui.core;
import bindbc.sdl;
import bindbc.imgui;

public import inui.core.window;
public import inui.core.app;
public import inui.core.path;
public import inui.core.settings;
public import inui.core.utils;
import inmath;

private {
    ulong SCREEN_TICKS_PER_FRAME = 0;

    ulong currTicks;
    ulong lastTicks;

    double lastTime;
    double currTime;
}

void inSetFPSCap(uint fps){
    if(fps > 1000){
        SCREEN_TICKS_PER_FRAME = 1;
    } else if(fps > 0){
        SCREEN_TICKS_PER_FRAME = 1000 / fps;
    }
    else {
        SCREEN_TICKS_PER_FRAME = 0;
    }
}

void inInitUI() {
    
    // Load and init SDL2
    loadSDL();
    SDL_Init(SDL_INIT_EVERYTHING);

    // Load imgui
    version(BindImGui_Dynamic) loadImGui();

    // Init settings store
    inSettingsLoad();
}

/**
    Updates time, called internally by inui
*/
void inUpdateTime() {
    lastTicks = currTicks;
    if (SDL_GetTicks64) currTicks = SDL_GetTicks64();
    else currTicks = cast(ulong)SDL_GetTicks();
    lastTime = currTime;
    currTime = cast(double)currTicks*0.001;
}

/**
    Returns the current timestep of the app
*/
double inGetTime() {
    return currTime;
}

/**
    Gets delta time
*/
double inGetDeltaTime() {
    return abs(lastTime-currTime);
}

void inWaitNextFrame(){
    if(SCREEN_TICKS_PER_FRAME > 0) {
        ulong frameTicks;
        if (SDL_GetTicks64) frameTicks = SDL_GetTicks64();
        else frameTicks = cast(ulong)SDL_GetTicks();

        frameTicks -= currTicks;

        if( frameTicks < SCREEN_TICKS_PER_FRAME )
        {
            //Wait remaining time
            SDL_Delay(cast(uint)(SCREEN_TICKS_PER_FRAME - frameTicks));
        }
    }
}